﻿using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using OpenHarbor.CQRS.Abstractions;

namespace OpenHarbor.CQRS.AspNetCore.Mvc;

[Produces("application/json")]
[ApiController, Route("api/query/[controller]")]
public class QueryController<TQuery, TQueryResult> : Controller
    where TQuery : class
{
    [HttpPost, QueryControllerAuthorization]
    public async Task<ActionResult<TQueryResult>> Handle([FromServices] IQueryHandler<TQuery, TQueryResult> handler, 
        [FromBody] TQuery query)
    {
        if (!ModelState.IsValid)
            return BadRequest(ModelState);


        return Ok(await handler.HandleAsync(query, this.Request.HttpContext.RequestAborted));
    }

    [HttpGet, QueryControllerAuthorization]
    public async Task<ActionResult<TQueryResult>> HandleGet([FromServices] IQueryHandler<TQuery, TQueryResult> handler,
        [FromQuery] TQuery query)
    {
        if (!ModelState.IsValid)
            return BadRequest(ModelState);


        return Ok(await handler.HandleAsync(query, this.Request.HttpContext.RequestAborted));
    }
}
